import setuptools

setuptools.setup(
    name="pho_mos",
    version="0.0.2",
    author="Vitaliy Rakitin",
    author_email="vitaliyrakitin@ya.ru",
    description="Photo mosaic",
    url="https://github.com/botanhuligan/pho_mos",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
